@@ -16,25 +16,20 @@
 #include	<fcntl.h>
 #include	<sys/types.h>
 #include	<sys/socket.h>
+#include	<sys/un.h>
+#include	<limits.h>
 #include        <netinet/in.h>
 #include	<netdb.h>
 #include	<string.h>
 #include	"builtin.h"
 #include	<errno.h>
 
-#define perror(s) FilePrintf(FileStderr, s ": %s\n", FileGetErrorMessage(errno))
-#ifdef HAVE_HSTRERROR
-#define herror(s) FilePrintf(FileStderr, s ": %s\n", hstrerror(h_errno))
-#else
-#define herror(s) FilePrintf(FileStderr, s ": network error %d\n", h_errno);
-#endif
-
 NamespacePtr SocketNamespace;
 Type	     *typeSockaddr;
 
-Value do_Socket_create (Value type);
-Value do_Socket_connect (Value s, Value host, Value port);
-Value do_Socket_bind (Value s, Value host, Value port);
+Value do_Socket_create (int num, Value *args);
+Value do_Socket_connect (int num, Value *args);
+Value do_Socket_bind (int numb, Value *args);
 Value do_Socket_listen (Value s, Value backlog);
 Value do_Socket_accept (Value s);
 Value do_Socket_shutdown (Value s, Value how);
@@ -55,12 +50,6 @@ import_Socket_namespace()
     };
 
     static const struct fbuiltin_1 funcs_1[] = {
-        { do_Socket_create, "create", "f", "i", "\n"
-	    " file create (int type)\n"
-	    "\n"
-	    " Create a socket where 'type' is one of:\n"
-	    "   SOCK_STREAM:	a stream socket.\n"
-	    "   SOCK_DGRAM:	a datagram socket.\n" },
         { do_Socket_accept, "accept", "f", "f", "\n"
 	    " file accept (file listen)\n"
 	    "\n"
@@ -87,19 +76,44 @@ import_Socket_namespace()
         { 0 }
     };
 
-    static const struct fbuiltin_3 funcs_3[] = {
-        { do_Socket_connect, "connect", "v", "fss", "\n"
-	    " void connect (file socket, string host, string port)\n"
+    static const struct fbuiltin_v funcs_v[] = {
+        { do_Socket_create, "create", "f", ".i", "\n"
+	    " file create ([int family], int type)\n"
 	    "\n"
-	    " Connect 'socket' to 'host', 'port'.\n" },
-        { do_Socket_bind, "bind", "v", "fss", "\n"
+	    " Create a socket where the optional 'family' is one of:\n"
+	    "   AF_UNIX:		Local communication.\n"
+	    "   AF_INET (default):	IPv4 Internet protocols.\n"
+	    "   AF_INET6:		IPv6 Internet protocols.\n"
+	    " and where 'type' is one of:\n"
+	    "   SOCK_STREAM:		a stream socket.\n"
+	    "   SOCK_DGRAM:		a datagram socket.\n" },
+        { do_Socket_bind, "bind", "v", "f.p", "\n"
 	    " void bind (file socket, string host, string port)\n"
+	    " void bind (file socket, string host, int port)\n"
 	    "\n"
-	    " Bind 'socket' to 'host', 'port'.\n" },
-        { 0 }
+	    " Bind AF_INET 'socket' to 'host', 'port'.\n"
+	    "\n"
+	    " void bind (file socket, string local_socket)\n"
+	    "\n"
+	    " Bind AF_UNIX 'socket' to 'localhost'.\n" },
+        { do_Socket_connect, "connect", "v", "f.p", "\n"
+	    " void connect (file socket, string host, string port)\n"
+	    " void connect (file socket, string host, int port)\n"
+	    "\n"
+	    " Connect AF_INET 'socket' to 'host', 'port'.\n"
+	    "\n"
+	    " void connect (file socket, string local_socket)\n"
+	    "\n"
+	    " Connect AF_UNIX 'socket' to 'local_socket'.\n" },
+	{ 0 }
     };
 
     static const struct ibuiltin ivars[] = {
+	{ AF_UNIX, "AF_UNIX", &SocketNamespace },
+	{ AF_INET, "AF_INET", &SocketNamespace },
+#ifdef AF_INET6
+	{ AF_INET6, "AF_INET6", &SocketNamespace },
+#endif
 	{ SOCK_STREAM, "SOCK_STREAM", &SocketNamespace },
 	{ SOCK_DGRAM, "SOCK_DGRAM", &SocketNamespace },
 	{ SHUT_RD, "SHUT_RD", &SocketNamespace },
@@ -133,7 +147,7 @@ import_Socket_namespace()
     BuiltinFuncs0 (&SocketNamespace, funcs_0);
     BuiltinFuncs1 (&SocketNamespace, funcs_1);
     BuiltinFuncs2 (&SocketNamespace, funcs_2);
-    BuiltinFuncs3 (&SocketNamespace, funcs_3);
+    BuiltinFuncsV (&SocketNamespace, funcs_v);
 
     BuiltinIntegers (ivars);
     BuiltinStrings (svars);
@@ -143,78 +157,184 @@ import_Socket_namespace()
 
 /* File::file do_Socket_create ({SOCK_STREAM,SOCK_DGRAM} type); */
 Value
-do_Socket_create (Value type)
+do_Socket_create (int num, Value *args)
 {
     ENTER ();
-    int itype, s;
-    itype = IntPart (type, "Illegal socket type");
+    int ifamily, itype, type_index, s;
+    Value ret;
+
+    if (num == 0 || num > 2) {
+	RaiseStandardException (exception_invalid_argument, 3,
+				NewStrString ("create must have one or two arguments"),
+				NewInt (0), NewInt (num));
+	RETURN (Void);
+    }
+
+    if (num > 1) {
+	ifamily = IntPart (args[0], "Illegal address family");
+	type_index = 1;
+    } else {
+	ifamily = AF_INET;
+	type_index = 0;
+    }
+    itype = IntPart (args[type_index], "Illegal socket type");
     if (aborting)
 	RETURN (Void);
-    s = socket (PF_INET, itype, 0);
+    s = socket (ifamily, itype, 0);
     if (s == -1)
 	RETURN (Void);
-    RETURN (FileCreate (s, FileReadable|FileWritable));
+    ret = FileCreate (s, FileReadable|FileWritable);
+    ret->file.sock_family = ifamily;
+    RETURN (ret);
 }
 
-static Bool address_lookup (Value hostname, Value portname,
-			    struct sockaddr_in *addr)
+typedef union {
+    struct sockaddr_in in;
+    struct sockaddr_un un;
+    struct sockaddr addr;
+    struct {
+	struct sockaddr_un un;
+	char path[PATH_MAX];
+    } align;
+} sockaddr_all_t;
+
+#define VerifyArgumentCount(arg, condition, error)			\
+if (! (condition)) {							\
+    RaiseStandardException (exception_invalid_argument, 3,    		\
+                            NewStrString (error), NewInt (0), NewInt (arg));	\
+}
+
+/* Supports the following args from both bind and connect:
+ *	(File::file s, String local_socket)
+ */
+static Bool address_lookup_af_unix (int num, Value *args,
+				    struct sockaddr_un *addr, socklen_t *len)
 {
-    struct hostent *host;
-    struct servent *port;
-    char *hostchars;
-    char *portchars;
-    char *endptr = 0;
-    long int portnum;
+    char *local_socket;
+    Value s = args[0];
 
-    hostchars = StrzPart (hostname, "invalid hostname");
-    if (!hostchars)
+    VerifyArgumentCount (num, num == 2,
+			 "must have 2 arguments for an AF_UNIX socket");
+    if (aborting)
 	return False;
-    portchars = StrzPart (portname, "invalid portname");
-    if (!portchars)
+
+    local_socket = StrzPart (args[1], "invalid local_socket");
+    if (!local_socket || *local_socket == '\0')
 	return False;
-    
-    if (*hostchars == '\0' || *portchars == '\0')
-	return False; /* FIXME: more here? */
 
-    addr->sin_family = AF_INET;
+    if (strlen (local_socket) > PATH_MAX)
+	return False;
 
-    /* host lookup */
-    host = gethostbyname (hostchars);
-    if (host == 0)
-    {
-	herror ("address_lookup");
-	return False; /* FIXME: more here? */
-    }
-    memcpy (&addr->sin_addr.s_addr, host->h_addr_list[0], sizeof addr->sin_addr.s_addr);
-
-    /* port lookup */
-    portnum = strtol (portchars, &endptr, /* base */ 10);
-    if (*endptr != '\0') /* non-numeric port specification */
-    {
-	/* FIXME: this should not always be "tcp"! */
-	port = getservbyname (portchars, "tcp");
-	if (port == 0)
-	    return False; /* FIXME: more here? */
-	addr->sin_port = port->s_port;
-    }
-    else
-    {
-	if (portnum <= 0 || portnum >= (1 << 16))
-	    return False; /* FIXME: more here? */
-	addr->sin_port = htons (portnum);
-    }
+    addr->sun_family = s->file.sock_family;
+    strcpy (addr->sun_path, local_socket);
+    *len = SUN_LEN (addr);
 
     return True;
 }
 
-/* void do_Socket_connect (File::file s, String host, String port); */
+/* Supports the following args from both bind and connect:
+ *	(File::file s, String host, String port)
+ *	(File::file s, String host, int port)
+ */
+static Bool address_lookup_af_inet (int num, Value *args,
+				    struct sockaddr_in *addr, socklen_t *len)
+{
+    Value host, port;
+    char *hostchars;
+    struct hostent *hostent;
+    long int portnum;
+    struct servent *portent;
+    char *endptr;
+    Value s = args[0];
+
+    VerifyArgumentCount (num, num == 3,
+			 "must have 3 arguments for an AF_INET socket");
+    if (aborting)
+	return False;
+
+    host = args[1];
+    port = args[2];
+
+    hostchars = StrzPart (host, "invalid hostname");
+    if (!hostchars || *hostchars == '\0')
+	return False;
+
+    if (ValueIsString (port)) {
+	char *portchars = StrzPart (port, "invalid port string");
+	if (!portchars || *portchars == '\0')
+	    return False;
+	portnum = strtol (portchars, &endptr, /* base */ 10);
+	if (*endptr != '\0') /* non-numeric port specification */
+	{
+	    /* FIXME: this should not always be "tcp"! */
+	    portent = getservbyname (portchars, "tcp");
+	    if (portent == 0)
+		return False; /* FIXME: more here? */
+	    addr->sin_port = portent->s_port;
+	}
+	if (portnum <= 0 || portnum >= (1 << 16))
+	    return False; /* FIXME: more here? */
+    } else {
+	portnum = IntPart (port, "invalid port value");
+	if (portnum <= 0 || portnum >= (1 << 16))
+	    return False; /* FIXME: more here? */
+    }
+
+    addr->sin_family = s->file.sock_family;
+    addr->sin_port = htons (portnum);
+
+    /* host lookup */
+    hostent = gethostbyname (hostchars);
+    if (hostent == 0)
+    {
+	return False; /* FIXME: more here? */
+    }
+
+    *len = sizeof (*addr);
+    memcpy (&addr->sin_addr.s_addr, hostent->h_addr_list[0],
+	    sizeof (addr->sin_addr.s_addr));
+
+    return True;
+}
+
+/* Supports the following args from both bind and connect:
+ *
+ *	(File::file s, String host, String port)
+ *	(File::file s, String host, int port)
+ *	(File::file s, String local_socket)
+ */
+static Bool address_lookup (int num, Value *args,
+			    sockaddr_all_t *addr, socklen_t *len)
+{
+    Value s = args[0];
+
+    switch (s->file.sock_family) {
+    case AF_UNIX:
+	return address_lookup_af_unix (num, args, &addr->un, len);
+    case AF_INET:
+	return address_lookup_af_inet (num, args, &addr->in, len);
+#ifdef AF_INET6
+    case AF_INET6:
+	/* FIXME */
+#endif
+    default:
+	return False;
+    }
+}
+
+/* void do_Socket_connect (File::file s, String host, String port);
+ * void do_Socket_connect (File::file s, String host, int port);
+ * void do_Socket_connect (File::file s, String local_socket;
+ */
 Value
-do_Socket_connect (Value s, Value host, Value port)
+do_Socket_connect (int num, Value *args)
 {
     ENTER ();
-    struct sockaddr_in addr;
+    sockaddr_all_t addr;
+    socklen_t len;
+    Value s = args[0];
 
-    if (!address_lookup (host, port, &addr))
+    if (!address_lookup (num, args, &addr, &len))
 	RETURN (Void);
 
     if (!running->thread.partial)
@@ -230,7 +350,7 @@ do_Socket_connect (Value s, Value host, Value port)
 			(char *) &one, sizeof (int));
 	}
 #endif
-	n = connect (s->file.fd, (struct sockaddr *) &addr, sizeof addr);
+	n = connect (s->file.fd, &addr.addr, len);
 	flags &= ~O_NONBLOCK;
 	fcntl (s->file.fd, F_SETFL, flags);
 	err = errno;
@@ -243,9 +363,9 @@ do_Socket_connect (Value s, Value host, Value port)
 	    }
 	    else
 	    {
-		RaiseStandardException (exception_io_error,
+		RaiseStandardException (exception_io_error, 3,
 					FileGetErrorMessage (err),
-					2, FileGetError (err),
+					FileGetError (err),
 					s);
 		RETURN (Void);
 	    }
@@ -261,14 +381,19 @@ do_Socket_connect (Value s, Value host, Value port)
     RETURN (Void);
 }
 
-/* void do_Socket_bind (File::file s, String host, String port); */
+/* void do_Socket_bind (File::file s, String host, String port);
+ * void do_Socket_bind (File::file s, String host, int port);
+ * void do_Socket_bind (File::file s, String local_socket;
+ */
 Value
-do_Socket_bind (Value s, Value host, Value port)
+do_Socket_bind (int num, Value *args)
 {
     ENTER ();
-    struct sockaddr_in addr;
+    sockaddr_all_t addr;
+    socklen_t len;
+    Value s = args[0];
 
-    if (!address_lookup (host, port, &addr))
+    if (!address_lookup (num, args, &addr, &len))
 	RETURN (Void);
 
 #ifdef SO_REUSEADDR
@@ -277,11 +402,11 @@ do_Socket_bind (Value s, Value host, Value port)
 	setsockopt (s->file.fd, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof (int));
     }
 #endif
-    if (bind (s->file.fd, (struct sockaddr *) &addr, sizeof addr) == -1)
+    if (bind (s->file.fd, &addr.addr, len) == -1)
     {
-	RaiseStandardException (exception_io_error,
+	RaiseStandardException (exception_io_error, 3,
 				FileGetErrorMessage (errno),
-				2, FileGetError (errno),
+				FileGetError (errno),
 				s);
 	RETURN (Void);
     }
@@ -330,9 +455,9 @@ do_Socket_accept (Value s)
 	}
 	else
 	{
-	    RaiseStandardException (exception_io_error,
+	    RaiseStandardException (exception_io_error, 3,
 				    FileGetErrorMessage (err),
-				    2, FileGetError (err),
+				    FileGetError (err),
 				    s);
 	    RETURN (Void);
 	}
@@ -377,9 +502,9 @@ do_Socket_gethostname (void)
 
     if (gethostname (hostname, sizeof (hostname)) == -1)
     {
-	RaiseStandardException (exception_io_error,
+	RaiseStandardException (exception_io_error, 3,
 				FileGetErrorMessage (errno),
-				2, FileGetError (errno),
+				FileGetError (errno),
 				Void);
 	RETURN (Void);
     }
@@ -399,9 +524,9 @@ do_Socket_getsockname (Value s)
 
     if (getsockname (s->file.fd, (struct sockaddr *) &addr, &len) == -1)
     {
-	RaiseStandardException (exception_io_error,
+	RaiseStandardException (exception_io_error, 3,
 				FileGetErrorMessage (errno),
-				2, FileGetError (errno),
+				FileGetError (errno),
 				s);
 	RETURN (Void);
     }
