@@ -43,28 +43,23 @@ PrettyProfNum (Value f, unsigned long i, int pad_left)
 	    FilePuts (f, " ");
 }
 	    
-static void PrettyParameters (Value f, Expr *e, Bool nest, ProfileData *pd);
-static void PrettyArrayInit (Value f, Expr *e, int level, Bool nest, ProfileData *pd);
-static void PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest, ProfileData *pd);
-static void PrettyBody (Value f, CodePtr code, int level, Bool nest, ProfileData *pd);
-static void PrettyDoc (Value f, int level, Value doc, ProfileData *pd);
+static void PrettyParameters (Value f, Expr *e, Bool nest);
+static void PrettyArrayInit (Value f, Expr *e, int level, Bool nest);
+static void PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest);
+static void PrettyBody (Value f, CodePtr code, int level, Bool nest);
+static void PrettyDoc (Value f, int level, Value doc);
 
 static void
-PrettyIndent (Value f, Expr *e, int level, ProfileData *pd)
+PrettyIndent (Value f, Expr *e, int level)
 {
     int	i;
     if (profiling)
     {
 	if (e)
 	{
-	    PrettyProfNum (f, e->base.sub_ticks, 1);
+	    PrettyProfNum (f, e->base.sub_ticks + e->base.ticks, 1);
 	    FilePuts (f, " ");
 	    PrettyProfNum (f, e->base.ticks, 1);
-	    if (pd)
-	    {
-		pd->sub += e->base.sub_ticks;
-		pd->self += e->base.ticks;
-	    }
 	}
 	else
 	    FilePuts (f, "                     ");
@@ -77,10 +72,10 @@ PrettyIndent (Value f, Expr *e, int level, ProfileData *pd)
 }
 
 static void
-PrettyBlock (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
+PrettyBlock (Value f, Expr *e, int level, Bool nest)
 {
     while (e->tree.left) {
-	PrettyStatement (f, e->tree.left, level, level, nest, pd);
+	PrettyStatement (f, e->tree.left, level, level, nest);
 	e = e->tree.right;
     }
 }
@@ -145,18 +140,18 @@ tokenToPrecedence (int token)
 }
 
 static void
-PrettyParameters (Value f, Expr *e, Bool nest, ProfileData *pd)
+PrettyParameters (Value f, Expr *e, Bool nest)
 {
     while (e) 
     {
 	if (e->tree.left->base.tag == DOTDOTDOT)
 	{
-	    PrettyExpr (f, e->tree.left->tree.left, -1, 0, nest, pd);
+	    PrettyExpr (f, e->tree.left->tree.left, -1, 0, nest);
 	    FilePuts (f, "...");
 	}
 	else
 	{
-	    PrettyExpr (f, e->tree.left, -1, 0, nest, pd);
+	    PrettyExpr (f, e->tree.left, -1, 0, nest);
 	}
 	e = e->tree.right;
 	if (e)
@@ -165,15 +160,15 @@ PrettyParameters (Value f, Expr *e, Bool nest, ProfileData *pd)
 }
 
 static void
-PrettyArrayInit (Value f, Expr *e, int level, Bool nest, ProfileData *pd);
+PrettyArrayInit (Value f, Expr *e, int level, Bool nest);
 
 static void
-PrettyArrayInits (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
+PrettyArrayInits (Value f, Expr *e, int level, Bool nest)
 {
     while (e)
     {
 	if (e->tree.left)
-	    PrettyArrayInit (f, e->tree.left, 0, nest, pd);
+	    PrettyArrayInit (f, e->tree.left, 0, nest);
 	e = e->tree.right;
 	if (e)
 	{
@@ -186,31 +181,31 @@ PrettyArrayInits (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
 }
 
 static void
-PrettyArrayInit (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
+PrettyArrayInit (Value f, Expr *e, int level, Bool nest)
 {
     switch (e->base.tag) {
     case OC:
 	FilePuts (f, "{ ");
-	PrettyArrayInits (f, e->tree.left, level, nest, pd);
+	PrettyArrayInits (f, e->tree.left, level, nest);
 	FilePuts (f, " }");
 	break;
     case DOTDOTDOT:
 	FilePuts (f, "...");
 	break;
     default:
-	PrettyExpr (f, e, -1, level, nest, pd);
+	PrettyExpr (f, e, -1, level, nest);
 	break;
     }
 }
 
 static void
-PrettyHashInit (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
+PrettyHashInit (Value f, Expr *e, int level, Bool nest)
 {
     while (e)
     {
-	PrettyExpr (f, e->tree.left->tree.left, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left->tree.left, -1, level, nest);
 	FilePuts (f, " => ");
-	PrettyExpr (f, e->tree.left->tree.right, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left->tree.right, -1, level, nest);
 	e = e->tree.right;
 	if (e)
 	    FilePuts (f, ", ");
@@ -218,13 +213,13 @@ PrettyHashInit (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
 }
 
 static void
-PrettyStructInit (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
+PrettyStructInit (Value f, Expr *e, int level, Bool nest)
 {
     while (e)
     {
 	FilePuts (f, AtomName (e->tree.left->tree.left->atom.atom));
 	FilePuts (f, " = ");
-	PrettyExpr (f, e->tree.left->tree.right, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left->tree.right, -1, level, nest);
 	e = e->tree.right;
 	if (e)
 	    FilePuts (f, ", ");
@@ -265,7 +260,7 @@ PrettyChar (Value f, int c)
 }
 
 static void
-PrettyDecl (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
+PrettyDecl (Value f, Expr *e, int level, Bool nest)
 {
     DeclListPtr	decl;
 
@@ -311,7 +306,7 @@ PrettyDecl (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
 	if (decl->init)
 	{
 	    FilePuts (f, " = ");
-	    PrettyExpr (f, decl->init, -1, level, nest, pd);
+	    PrettyExpr (f, decl->init, -1, level, nest);
 	}
 	if (decl->next)
 	    FilePuts (f, ",");
@@ -323,7 +318,7 @@ PrettyDecl (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
 }
 
 void
-PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData *pd)
+PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest)
 {
     int	selfPrec;
 
@@ -337,19 +332,19 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData
 	FilePuts (f, AtomName (e->atom.atom));
 	break;
     case VAR:
-	PrettyDecl (f, e, level, nest, pd);
+	PrettyDecl (f, e, level, nest);
 	break;
     case OP:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	FilePuts (f, " (");
 	if (e->tree.right)
-	    PrettyParameters (f, e->tree.right, nest, pd);
+	    PrettyParameters (f, e->tree.right, nest);
 	FilePuts (f, ")");
 	break;
     case OS:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	FilePuts (f, "[");
-	PrettyParameters (f, e->tree.right, nest, pd);
+	PrettyParameters (f, e->tree.right, nest);
 	FilePuts (f, "]");
 	break;
     case NEW:
@@ -357,31 +352,31 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData
 	if (e->tree.left)
 	{
 	    FilePuts (f, " ");
-	    PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	    PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	}
 	break;
     case ARRAY:
 	FilePuts (f, "{ ");
-	PrettyArrayInits (f, e->tree.left, level, nest, pd);
+	PrettyArrayInits (f, e->tree.left, level, nest);
 	FilePuts (f, " }");
 	break;
     case COMP:
 	FilePuts (f, "{ ");
 	FilePuts (f, "[");
-	PrettyExpr (f, e->tree.left->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left->tree.left, selfPrec, level, nest);
 	FilePuts (f, "] ");
 	if (e->tree.right->base.tag == OC)
-	    PrettyStatement (f, e->tree.right, level + 1, level, nest, pd);
+	    PrettyStatement (f, e->tree.right, level + 1, level, nest);
 	else
 	{
 	    FilePuts (f, "= ");
-	    PrettyExpr (f, e->tree.right, selfPrec, level, nest, pd);
+	    PrettyExpr (f, e->tree.right, selfPrec, level, nest);
 	}
 	FilePuts (f, " }");
 	break;
     case HASH:
 	FilePuts (f, "{ ");
-	PrettyHashInit (f, e->tree.left, level, nest, pd);
+	PrettyHashInit (f, e->tree.left, level, nest);
 	FilePuts (f, " }");
 	break;
     case ANONINIT:
@@ -389,14 +384,14 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData
 	break;
     case STRUCT:
 	FilePuts (f, "{ ");
-	PrettyStructInit (f, e->tree.left, level, nest, pd);
+	PrettyStructInit (f, e->tree.left, level, nest);
 	FilePuts (f, " }");
 	break;
     case UNION:
 	if (e->tree.right)
 	{
 	    FilePrintf (f, "(%T.%A) ", e->base.type, e->tree.left->atom.atom);
-	    PrettyExpr (f, e->tree.right, selfPrec, level, nest, pd);
+	    PrettyExpr (f, e->tree.right, selfPrec, level, nest);
 	}
 	else
 	{
@@ -472,7 +467,7 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData
     case ASSIGNAND:
     case ASSIGNOR:
     case COMMA:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	switch (e->base.tag) {
 	case PLUS:	FilePuts (f, " + "); break;
 	case MINUS:	FilePuts (f, " - "); break;
@@ -511,10 +506,10 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData
 	case ASSIGNOR:	FilePuts (f, " ||= "); break;
 	case COMMA:	FilePuts (f, ", "); break;
 	}
-	PrettyExpr (f, e->tree.right, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.right, selfPrec, level, nest);
 	break;
     case FACT:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	FilePuts (f, "!"); 
 	break;
     case LNOT:
@@ -523,7 +518,7 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData
     case INC:
     case DEC:
 	if (e->tree.right)
-	    PrettyExpr (f, e->tree.right, selfPrec, level, nest, pd);
+	    PrettyExpr (f, e->tree.right, selfPrec, level, nest);
 	switch (e->base.tag) {
 	case LNOT:	FilePuts (f, "~"); break;
 	case UMINUS:	FilePuts (f, "-"); break;
@@ -532,49 +527,49 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData
 	case DEC:	FilePuts (f, "--"); break;
 	}
 	if (e->tree.left)
-	    PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	    PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	break;
     case STAR:
 	FilePuts (f, "*");
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	break;
     case AMPER:
 	FilePuts (f, "&");
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	break;
     case COLONCOLON:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	FilePuts (f, "::");
 	FilePuts (f, AtomName (e->tree.right->atom.atom));
 	break;
     case DOT:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	FileOutput (f, '.');
 	FilePuts (f, AtomName (e->tree.right->atom.atom));
 	break;
     case ARROW:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	FilePuts (f, "->");
 	FilePuts (f, AtomName (e->tree.right->atom.atom));
 	break;
     case QUEST:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	FilePuts (f, " ? ");
-	PrettyExpr (f, e->tree.right->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.right->tree.left, selfPrec, level, nest);
 	FilePuts (f, " : ");
-	PrettyExpr (f, e->tree.right->tree.right, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.right->tree.right, selfPrec, level, nest);
 	break;
     case DOLLAR:
 	if (e->tree.left)
 	{
 	    FilePuts (f, "$");
-	    PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	    PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	}
 	else
 	    FilePuts (f, ".");
 	break;
     case EXPR:
-	PrettyExpr (f, e->tree.left, selfPrec, level, nest, pd);
+	PrettyExpr (f, e->tree.left, selfPrec, level, nest);
 	break;
     }
     if (selfPrec < parentPrec)
@@ -582,16 +577,16 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest, ProfileData
 }
 
 static void
-_PrettyCatch (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
+_PrettyCatch (Value f, Expr *e, int level, Bool nest)
 {
     CodePtr	catch;
     Atom	name;
     
     if (!e)
 	return;
-    _PrettyCatch (f, e->tree.left, level, nest, pd);
+    _PrettyCatch (f, e->tree.left, level, nest);
     if (nest)
-	PrettyIndent (f, 0, level, pd);
+	PrettyIndent (f, 0, level);
     e = e->tree.right;
     catch = e->code.code;
     if (catch->base.name->base.tag == COLONCOLON)
@@ -599,98 +594,101 @@ _PrettyCatch (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
     else
 	name = catch->base.name->atom.atom;
     FilePuts (f, "catch ");
-    PrettyExpr (f, catch->base.name, 0, level, nest, pd);
+    PrettyExpr (f, catch->base.name, 0, level, nest);
     FilePuts (f, " ");
-    PrettyBody (f, catch, level, nest, pd);
+    PrettyBody (f, catch, level, nest);
     FilePuts (f, "\n");
 }
 
+static void
+PrintArgs (Value f, ArgType *args);
+    
 void
-PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest, ProfileData *pd)
+PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest)
 {
     switch (e->base.tag) {
     case EXPR:
-	PrettyIndent (f, e, level, pd);
-	PrettyExpr (f, e->tree.left, -1, level, nest, pd);
+	PrettyIndent (f, e, level);
+	PrettyExpr (f, e->tree.left, -1, level, nest);
 	FilePuts (f, ";\n");
 	break;
     case IF:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "if (");
-	PrettyExpr (f, e->tree.left, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left, -1, level, nest);
 	FilePuts (f, ")\n");
 	if (nest)
-	    PrettyStatement (f, e->tree.right, level+1, level, nest, pd);
+	    PrettyStatement (f, e->tree.right, level+1, level, nest);
 	break;
     case ELSE:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "if (");
-	PrettyExpr (f, e->tree.left, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left, -1, level, nest);
 	FilePuts (f, ")\n");
 	if (nest)
 	{
-	    PrettyStatement (f, e->tree.right->tree.left, level+1, level, nest, pd);
-	    PrettyIndent (f, 0, level, pd);
+	    PrettyStatement (f, e->tree.right->tree.left, level+1, level, nest);
+	    PrettyIndent (f, 0, level);
 	    FilePuts (f, "else\n");
-	    PrettyStatement (f, e->tree.right->tree.right, level+1, level, nest, pd);
+	    PrettyStatement (f, e->tree.right->tree.right, level+1, level, nest);
 	}
 	break;
     case WHILE:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "while (");
-	PrettyExpr (f, e->tree.left, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left, -1, level, nest);
 	FilePuts (f, ")\n");
 	if (nest)
-	    PrettyStatement (f, e->tree.right, level+1, level, nest, pd);
+	    PrettyStatement (f, e->tree.right, level+1, level, nest);
 	break;
     case OC:
-	PrettyIndent (f, 0, blevel, pd);
+	PrettyIndent (f, 0, blevel);
 	FilePuts (f, "{\n");
-	PrettyBlock (f, e, blevel + 1, nest, pd);
-	PrettyIndent (f, 0, blevel, pd);
+	PrettyBlock (f, e, blevel + 1, nest);
+	PrettyIndent (f, 0, blevel);
 	FilePuts (f, "}\n");
 	break;
     case DO:
-	PrettyIndent (f, 0, level, pd);
+	PrettyIndent (f, 0, level);
 	FilePuts (f, "do\n");
 	if (nest)
-	    PrettyStatement (f, e->tree.left, level+1, level, nest, pd);
-	PrettyIndent (f, e, level, pd);
+	    PrettyStatement (f, e->tree.left, level+1, level, nest);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "while (");
-	PrettyExpr (f, e->tree.right, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.right, -1, level, nest);
 	FilePuts (f, ");\n");
 	break;
     case FOR:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "for (");
 	if (e->tree.left->tree.left)
-	    PrettyExpr (f, e->tree.left->tree.left, -1, level, nest, pd);
+	    PrettyExpr (f, e->tree.left->tree.left, -1, level, nest);
 	if (e->tree.left->base.tag == SEMI)
 	    FilePuts (f, ";");
 	if (e->tree.left->tree.right->tree.left)
 	{
 	    if (e->tree.left->base.tag == SEMI)
 		FilePuts (f, " ");
-	    PrettyExpr (f, e->tree.left->tree.right->tree.left, -1, level, nest, pd);
+	    PrettyExpr (f, e->tree.left->tree.right->tree.left, -1, level, nest);
 	}
 	FilePuts (f, ";");
 	if (e->tree.left->tree.right->tree.right->tree.left)
 	{
 	    FilePuts (f, " ");
-	    PrettyExpr (f, e->tree.left->tree.right->tree.right->tree.left, -1, level, nest, pd);
+	    PrettyExpr (f, e->tree.left->tree.right->tree.right->tree.left, -1, level, nest);
 	}
 	FilePuts (f, ")\n");
 	if (nest)
-	    PrettyStatement (f, e->tree.right, level+1, level, nest, pd);
+	    PrettyStatement (f, e->tree.right, level+1, level, nest);
 	break;
     case SWITCH:
     case UNION:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	if (e->base.tag == SWITCH)
 	    FilePuts (f, "switch (");
 	else
 	    FilePuts (f, "union switch (");
-	PrettyExpr (f, e->tree.left, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left, -1, level, nest);
 	FilePuts (f, ")");
 	if (nest)
 	{
@@ -699,52 +697,63 @@ PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest, ProfileData
 	    FilePuts (f, " {\n");	    
 	    while (block)
 	    {
-		PrettyIndent (f, 0, level, pd);
+		PrettyIndent (f, 0, level);
 		if (block->tree.left->tree.left)
 		{
 		    FilePuts (f, "case ");
-		    PrettyExpr (f, block->tree.left->tree.left, -1, level, nest, pd);
+		    PrettyExpr (f, block->tree.left->tree.left, -1, level, nest);
 		}
 		else
 		    FilePuts (f, "default");
 		FilePuts (f, ":\n");
-		PrettyBlock (f, block->tree.left->tree.right, level+1, nest, pd);
+		PrettyBlock (f, block->tree.left->tree.right, level+1, nest);
 		block = block->tree.right;
 	    }
-	    PrettyIndent (f, 0, level, pd);
+	    PrettyIndent (f, 0, level);
 	    FilePuts (f, "}");
 	}
         FilePuts (f, "\n");
 	break;
     case SEMI:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, ";\n");
 	break;
     case BREAK:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "break;\n");
 	break;
     case CONTINUE:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "continue;\n");
 	break;
     case RETURNTOK:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "return ");
-	PrettyExpr (f, e->tree.right, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.right, -1, level, nest);
 	FilePuts (f, ";\n");
 	break;
     case FUNC:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	{
 	    DeclListPtr	    decl = e->decl.decl;
 	    ExprPtr	    init = decl->init;
-	    CodePtr	    code = init->code.code;
-
-	    PrettyCode (f, code, decl->name, 
-			e->decl.class,
-			e->decl.publish,
-			level, nest);
+	    if (init)
+	    {
+		CodePtr	    code = init->code.code;
+    
+		PrettyCode (f, code, decl->name, 
+			    e->decl.class,
+			    e->decl.publish,
+			    level, nest);
+	    }
+	    else
+	    {
+		Type	*t = e->decl.type;
+		FilePrintf (f, "%p%k%T %A ", e->decl.publish,
+			    e->decl.class, t->func.ret, decl->name);
+		PrintArgs (f, t->func.args);
+		FilePuts (f, ";");
+	    }
 	}
 	FilePuts (f, "\n");
 	break;
@@ -752,55 +761,55 @@ PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest, ProfileData
 	e = e->tree.left;
 	/* fall through */
     case VAR:
-	PrettyIndent (f, e, level, pd);
-	PrettyDecl (f, e, level, nest, pd);
+	PrettyIndent (f, e, level);
+	PrettyDecl (f, e, level, nest);
         FilePuts (f, ";\n");
 	break;
     case NAMESPACE:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "namespace ");
-	PrettyExpr (f, e->tree.left, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left, -1, level, nest);
 	FilePuts (f, "\n");
-	PrettyStatement (f, e->tree.right, level + 1, level, nest, pd);
+	PrettyStatement (f, e->tree.right, level + 1, level, nest);
 	break;
     case IMPORT:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePrintf (f, "%pimport ", e->tree.right->decl.publish);
-	PrettyExpr (f, e->tree.left, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left, -1, level, nest);
 	FilePuts (f, ";\n");
 	break;
     case TWIXT:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "twixt (");
-	PrettyExpr (f, e->tree.left->tree.left, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left->tree.left, -1, level, nest);
 	FilePuts (f, "; ");
-	PrettyExpr (f, e->tree.left->tree.right, -1, level, nest, pd);
+	PrettyExpr (f, e->tree.left->tree.right, -1, level, nest);
 	FilePuts (f, ")\n");
 	if (nest)
-	    PrettyStatement (f, e->tree.right->tree.left, level+1, level, nest, pd);
+	    PrettyStatement (f, e->tree.right->tree.left, level+1, level, nest);
 	break;
     case CATCH:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePuts (f, "try");
 	if (nest)
 	{
 	    FilePuts (f, "\n");
-	    PrettyStatement (f, e->tree.right, level+1, level, nest, pd);
+	    PrettyStatement (f, e->tree.right, level+1, level, nest);
 	}
 	else
 	    FilePuts (f, " ");
-	_PrettyCatch (f, e->tree.left, level, nest, pd);
+	_PrettyCatch (f, e->tree.left, level, nest);
 	break;
     case RAISE:
-	PrettyIndent (f, e, level, pd);
+	PrettyIndent (f, e, level);
 	FilePrintf (f, "raise %A (", e->tree.left->atom.atom);
 	if (e->tree.right)
-	    PrettyParameters (f, e->tree.right, nest, pd);
+	    PrettyParameters (f, e->tree.right, nest);
 	FilePuts (f, ");\n");
 	break;
     case DOLLAR:
-	PrettyIndent (f, e, level, pd);
-	PrettyExpr (f, e->tree.left, -1, level, nest, pd);
+	PrettyIndent (f, e, level);
+	PrettyExpr (f, e->tree.left, -1, level, nest);
 	FilePuts (f, "\n");
 	break;
     }
@@ -824,20 +833,20 @@ PrintArgs (Value f, ArgType *args)
 }
 
 static void
-PrettyDoc (Value f, int level, Value doc, ProfileData *pd)
+PrettyDoc (Value f, int level, Value doc)
 {
     char	*s = StringChars (&doc->string);
     long    	len = doc->string.length;
     unsigned	c;
     Bool	newline = False;
 
-    PrettyIndent (f, 0, level, pd);
+    PrettyIndent (f, 0, level);
     FilePuts (f, "/""*");
     while ((s = StringNextChar (s, &c, &len)))
     {
 	if (newline)
 	{
-	    PrettyIndent (f, 0, level, pd);
+	    PrettyIndent (f, 0, level);
 	    FilePuts (f, " *");
 	    newline = False;
 	}
@@ -847,20 +856,20 @@ PrettyDoc (Value f, int level, Value doc, ProfileData *pd)
     }
     if (newline)
     {
-	PrettyIndent (f, 0, level, pd);
+	PrettyIndent (f, 0, level);
 	FileOutput (f, ' ');
     }
     FilePuts (f, "*""/");
 }
 
 static void
-PrettyBody (Value f, CodePtr code, int level, Bool nest, ProfileData *pd)
+PrettyBody (Value f, CodePtr code, int level, Bool nest)
 {
     PrintArgs (f, code->base.args);
     if (code->base.doc != Void)
     {
 	FilePuts (f, "\n");
-	PrettyDoc (f, level + 1, code->base.doc, pd);
+	PrettyDoc (f, level + 1, code->base.doc);
     }
     if (nest)
     {
@@ -871,10 +880,10 @@ PrettyBody (Value f, CodePtr code, int level, Bool nest, ProfileData *pd)
 	else
 	{
 	    FilePuts (f, "\n");
-	    PrettyIndent (f, 0, level, pd);
+	    PrettyIndent (f, 0, level);
 	    FilePuts (f, "{\n");
-	    PrettyBlock (f, code->func.code, level + 1, nest, pd);
-	    PrettyIndent (f, 0, level, pd);
+	    PrettyBlock (f, code->func.code, level + 1, nest);
+	    PrettyIndent (f, 0, level);
 	    FilePuts (f, "}");
 	}
     }
@@ -886,19 +895,26 @@ void
 PrettyCode (Value f, CodePtr code, Atom name, Class class, Publish publish,
 	   int level, Bool nest)
 {
-    ProfileData	pd;
-    pd.sub = pd.self = 0;
     if (name)
 	FilePrintf (f, "%p%k%T %A ", publish, class, code->base.type, name);
     else
 	FilePrintf (f, "%tfunc", code->base.type);
-    PrettyBody (f, code, level, nest, &pd);
+    PrettyBody (f, code, level, nest);
     if (!code->base.builtin && nest && profiling)
     {
+	double_digit	sub = 0, self = 0;
+	if (code->func.body.obj) {
+	    sub += code->func.body.obj->sub_ticks;
+	    self += code->func.body.obj->ticks;
+	}
+	if (code->func.staticInit.obj) {
+	    sub += code->func.staticInit.obj->sub_ticks;
+	    self += code->func.staticInit.obj->ticks;
+	}
 	FilePuts (f, "\n---------------------\n");
-	PrettyProfNum (f, pd.sub, 1);
+	PrettyProfNum (f, sub + self, 1);
 	FilePuts (f, " ");
-	PrettyProfNum (f, pd.self, 1);
+	PrettyProfNum (f, self, 1);
 	if (name)
 	    FilePrintf (f, ": %A\n", name);
 	else
@@ -909,7 +925,7 @@ PrettyCode (Value f, CodePtr code, Atom name, Class class, Publish publish,
 void
 PrettyStat (Value f, Expr *e, Bool nest)
 {
-    PrettyStatement (f, e, 1, 1, nest, 0);
+    PrettyStatement (f, e, 1, 1, nest);
 }
 
 void
@@ -939,8 +955,8 @@ doPrettyPrint (Value f, Publish publish, SymbolPtr symbol, int level, Bool nest)
     if (!symbol)
 	return;
     if (profiling)
-	FilePuts (f, "    called(ms)   self(ms)\n");
-    PrettyIndent (f, 0, level, 0);
+	FilePuts (f, "     total(ms)   self(ms)\n");
+    PrettyIndent (f, 0, level);
     switch (symbol->symbol.class) {
     case class_const:
     case class_global:
@@ -967,7 +983,7 @@ doPrettyPrint (Value f, Publish publish, SymbolPtr symbol, int level, Bool nest)
 	{
 	    FilePuts (f, " {\n");
 	    PrintNamespace (f, symbol->namespace.namespace, level + 1);
-	    PrettyIndent (f, 0, level, 0);
+	    PrettyIndent (f, 0, level);
 	    FilePuts (f, "}\n");
 	}
 	else
@@ -980,7 +996,7 @@ doPrettyPrint (Value f, Publish publish, SymbolPtr symbol, int level, Bool nest)
 	if (symbol->exception.doc != Void)
 	{
 	    FilePuts (f, "\n");
-	    PrettyDoc (f, level + 1, symbol->exception.doc, 0);
+	    PrettyDoc (f, level + 1, symbol->exception.doc);
 	}
 	FilePuts (f, ";\n");
 	break;
