@@ -440,6 +440,7 @@ typedef struct _typeStruct {
     TypeBase	    base;
     StructTypePtr   structs;
     Bool	    enumeration;
+    TypePtr	    left, right;
 } TypeStruct;    
 
 typedef struct _typeElt {
@@ -483,6 +484,7 @@ extern Type	    *typePrim[rep_void + 1];
 
 Type	*NewTypeName (ExprPtr expr, SymbolPtr name);
 Type	*NewTypeRef (Type *ref, Bool pointer);
+Type	*NewTypePlus (Type *left, Type *right);
 Type	*NewTypePointer (Type *ref);
 Type	*NewTypeFunc (Type *ret, ArgType *args);
 Type	*NewTypeArray (Type *type, ExprPtr dimensions, Bool resizable);
@@ -643,6 +645,7 @@ typedef struct _file {
     int		    error;
     FileChainPtr    input;
     FileChainPtr    output;
+    int		    sock_family;
 } File;
 
 #define FileBufferSize	4096
@@ -741,10 +744,9 @@ typedef struct _continuation {
 } Continuation;
 
 typedef enum _ThreadState {
-    ThreadRunning = 0,
-    ThreadSuspended = 1,
-    ThreadInterrupted = 2,
-    ThreadFinished = 4
+    ThreadRunning,
+    ThreadSuspended,
+    ThreadFinished
 } ThreadState;
 
 typedef struct _thread {
@@ -1045,7 +1047,7 @@ extern Value	Blank, Elementless, Void, TrueVal, FalseVal;
 # define False(v)	((v) != TrueVal)
 
 Value	FileGetError (int err);
-char	*FileGetErrorMessage (int err);
+Value	FileGetErrorMessage (int err);
 int	FileInput (Value);
 int	FileOutput (Value, char);
 void	FileUnput (Value, unsigned char);
